"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.setActionTypes = exports.SeedActions = exports.SeedActionType = void 0;
const tslib_1 = require("tslib");
const path_1 = tslib_1.__importDefault(require("path"));
const seeds_1 = require("../../seeds");
const store_1 = require("../../store");
const types_1 = require("../../types");
const utils_1 = require("../../utils");
const tasks_1 = require("../tasks");
const fileSystem_1 = require("./fileSystem");
const git_1 = require("./git");
/**
 * For now I'm going to have a remote (git) and download (tar)
 */
var SeedActionType;
(function (SeedActionType) {
    SeedActionType["create"] = "Creating from seed";
    SeedActionType["createFromDownload"] = "Creating from downloaded seed";
    SeedActionType["createFromRemote"] = "Creating from remote seed";
    SeedActionType["createFromLocal"] = "Creating from local seed";
    SeedActionType["configure"] = "Configuring Seed";
    SeedActionType["setEnvVars"] = "Writing environment variables";
})(SeedActionType = exports.SeedActionType || (exports.SeedActionType = {}));
var SeedActions;
(function (SeedActions) {
    SeedActions.createFromDownload = async (directory, seed) => {
        const response = await (0, tasks_1.downloadSeed)(seed);
        await (0, tasks_1.extractSeed)(seed, response.body, directory);
    };
    SeedActions.createFromRemote = async (directory, seed) => {
        await git_1.GitActions.clone(directory, seed);
    };
    SeedActions.createFromLocal = async (directory, seed) => {
        fileSystem_1.FileSystemActions.createDirectory(directory);
        await fileSystem_1.FileSystemActions.copyDirectoryContentsTo(seed.baseDir, directory);
    };
    SeedActions.create = async (directory, seed, seedDelivery) => {
        if (seed.type === seeds_1.SeedSource.remote) {
            if (store_1.configStore.values.external ||
                seedDelivery === seeds_1.SeedDelivery.download ||
                seed.tags?.includes(types_1.DslTag)) {
                return SeedActions.createFromDownload(directory, seed);
            }
            return SeedActions.createFromRemote(directory, seed);
        }
        if (seed.type === seeds_1.SeedSource.local) {
            return SeedActions.createFromLocal(directory, seed);
        }
        throw new Error(`Unknown seed type. Seed: ${seed}`);
    };
    SeedActions.configure = async (answers, config, plop) => {
        const { seed } = config;
        if (!seed) {
            throw new Error(`Invalid seed information, unable to configure. Seed: ${seed}`);
        }
        await seed.configure(answers, config, plop);
    };
    SeedActions.setEnvVars = async (directory, envVars) => {
        if (!(0, utils_1.directoryExists)(directory)) {
            throw new Error(`Target directory does not exist: ${directory}`);
        }
        const configFiles = [`.env.development`, `.env.production`].map((file) => path_1.default.join(directory, file));
        const { dsName, pkgScope, pkgName, workspaceDirectory, workspaceName = pkgScope, workspaceId, managementToken, forceOverwrite, } = envVars;
        const fileContents = [
            `# These environment variables will made available during build steps.`,
            `# Do NOT commit this file to source control`,
            `GENESISLCAP_DS_NAME='${dsName}'`,
            `GENESISLCAP_PKG_SCOPE='${pkgScope}'`,
            `GENESISLCAP_PKG_NAME='${pkgName}'`,
            `GENESISLCAP_WORKSPACE_DIRECTORY='${workspaceDirectory}'`,
            `GENESISLCAP_WORKSPACE_NAME='${workspaceName}'`,
            `GENESISLCAP_WORKSPACE_ID='${workspaceId}'`,
            `GENESISLCAP_TARGET_DIRECTORY='${directory}'`,
            `GENESISLCAP_MANAGEMENT_TOKEN='${managementToken}'`,
            `GENESISLCAP_FORCE_OVERWRITE='${forceOverwrite}'`,
        ].join('\n') + '\n';
        configFiles.forEach((file) => fileSystem_1.FileSystemActions.writeFile(file, fileContents));
    };
})(SeedActions = exports.SeedActions || (exports.SeedActions = {}));
const setActionTypes = (plop) => {
    plop.setActionType(SeedActionType.create, async (answers, config) => {
        const { directory, seed, seedDelivery } = config;
        await SeedActions.create(directory, seed, seedDelivery);
        return `'${seed.name}'`;
    });
    plop.setActionType(SeedActionType.createFromDownload, async (answers, config) => {
        const { directory, seed } = config;
        await SeedActions.createFromDownload(directory, seed);
        return ''; // < There is a typing issue with Plop, these need to return a string
    });
    plop.setActionType(SeedActionType.createFromRemote, async (answers, config) => {
        const { directory, seed } = config;
        await SeedActions.createFromRemote(directory, seed);
        return ''; // < There is a typing issue with Plop, these need to return a string
    });
    plop.setActionType(SeedActionType.createFromLocal, async (answers, config) => {
        const { directory, seed } = config;
        await SeedActions.createFromLocal(directory, seed);
        return ''; // < There is a typing issue with Plop, these need to return a string
    });
    plop.setActionType(SeedActionType.configure, async (answers, config) => {
        await SeedActions.configure(answers, config, plop);
        return ''; // < There is a typing issue with Plop, these need to return a string
    });
    /**
     * Might merge with above
     */
    plop.setActionType(SeedActionType.setEnvVars, async (answers, config) => {
        const { directory } = config;
        const { dsName, pkgScope, pkgName, workspaceDirectory, workspaceName = pkgScope, workspaceId, managementToken, forceOverwrite, } = answers;
        await SeedActions.setEnvVars(directory, {
            dsName,
            pkgScope,
            pkgName,
            workspaceDirectory,
            workspaceName,
            workspaceId,
            managementToken,
            forceOverwrite,
        });
        return ''; // < There is a typing issue with Plop, these need to return a string
    });
};
exports.setActionTypes = setActionTypes;
